/*
	Programme de chenillard illustrant l'utilisation d'un timer comme base de temps.
	Les LEDs sont connectes entre les pins du port D et la masse (=> allumes lorque
	l'I/O est  1)
	
	Le principe est de faire "tourner" le timer 0, et de rcuprer l'interruption
	 chaque passage de 255  0.
	
	Etant donn qu'on veut obtenir une cadence de dfilement visible (envrion 2/10e
	de seconde, il faut cumuler plusieurs divisions du temps.
	1/ 	on va appliquer un prescale de 1024  l'horloge processeur. Vu qu'elle
		est  8 MHz, la priode de base est de 125ns. Le precale va donc tablir la
		priode de l'incrment du compteur  125nS * 1024, soit 128s. Un tour complet
		de compteur va donc durer 128s * 256, soit 33ms environ. C'est la priode avec
		laquelle l'interruption sera dclenche
	2/ 	on va changer l'tat des sorties toutes les n interruptions, de manire  avoir
		une cadence de dfilement visible

	Cible : ATMega8
	Quartz : 8MHz
	Compilateur : avr-gcc (WInAVR)
*/

#include <avr/io.h> 
#include <avr/interrupt.h>

// compteur du nombre d'interruptions timer
volatile int tmr_cnt = 0 ;

// vu le prescaling et l'tendue du compteur, un intervalle de 100ms correspond  3 interruptions
#define _100_MS		 (3 * 64)
#define _1_S		 (30 * 4)

#define DIGIT_0 	0b00111111
#define DIGIT_1 	0b00000110
#define DIGIT_2 	0b01011011
#define DIGIT_3 	0b01001111
#define DIGIT_4		0b01100110
#define DIGIT_5 	0b01101101
#define DIGIT_6 	0b01111101
#define DIGIT_7 	0b00000111
#define DIGIT_8 	0b01111111
#define DIGIT_9 	0b01101111
#define DIGIT_A 	0b01110111
#define DIGIT_B 	0b01111100
#define DIGIT_b 	0b01111100
#define DIGIT_C 	0b00111001
#define DIGIT_d 	0b01011110
#define DIGIT_D 	0b01011110
#define DIGIT_E 	0b01111001
#define DIGIT_F 	0b01110001
#define DIGIT_g 	0b01111101
#define DIGIT_H 	0b01110110
#define DIGIT_h 	0b01110100
#define DIGIT_L 	0b00111000
#define DIGIT_l 	DIGIT_1
#define DIGIT_O 	DIGIT_0
#define DIGIT_o 	0b01011100
#define DIGIT_r 	0b01010000
#define DIGIT_W 	0b00111110
#define DIGIT_w 	0b00011100
#define DIGIT_sp	0b00000000
#define DIGIT_dp	0b10000000

unsigned char digits[] = {
	DIGIT_0,
	DIGIT_1,
	DIGIT_2,
	DIGIT_3,
	DIGIT_4,
	DIGIT_5,
	DIGIT_6,
	DIGIT_7,
	DIGIT_8,
	DIGIT_9, 
	DIGIT_A,
	DIGIT_B,
	DIGIT_C,
	DIGIT_D,
	DIGIT_E,
	DIGIT_F	
} ;

unsigned char msg[] = {
	DIGIT_H,
	DIGIT_E,
	DIGIT_L,
	DIGIT_L,
	DIGIT_O,
	DIGIT_sp,
	DIGIT_W,
	DIGIT_o,
	DIGIT_r,
	DIGIT_l,
	DIGIT_d,
	DIGIT_dp,
	DIGIT_dp,
	DIGIT_dp,
	DIGIT_sp,
	DIGIT_sp,
	DIGIT_sp,
	DIGIT_sp
} ;

unsigned char *p = msg ;

unsigned char digit_num = 0 ;
unsigned char digit_sel ;
unsigned char digit_value[4] = {DIGIT_sp, DIGIT_sp, DIGIT_sp, DIGIT_sp} ;
 
/*
	Interrupt handler pour l'overflow du timer 0 (T=33ms)
*/
SIGNAL (SIG_OVERFLOW0) {
	// A chaque interruption, on allume un des digits en activant les segments correspondants
	// au motif qu'il doit reproduire via le port A et en activant sa cathode via le port B.
	// Vue la frquence de balayage, l'oeil a l'illusion que les 4 digits sont allums en permanence 
	PORTA = digit_value[digit_num] ;
	PORTB = digit_sel ;
	
	digit_num = (digit_num + 1) & 0x03 ;
	digit_sel = 1 << digit_num ;
	

	// si on est arriv au nombre ncessaire  l'obtention du dlai de dfilement souhait,
	// on modifie le contenu du buffer reprsentant le message  afficher
	if (tmr_cnt == 0) {
		digit_value[3] = digit_value[2] ;
		digit_value[2] = digit_value[1] ;
		digit_value[1] = digit_value[0] ;
		digit_value[0] = *p++ ;
		
		if (p - msg == sizeof(msg)) p = msg ;
	}
	
	// mise  jour du compteur d'interruptions pour la gestion du dfilement
	tmr_cnt =  (tmr_cnt + 1) % (2 * _1_S) ;
}

/*
	Initialisation des ports
*/ 
void port_init(void) {
 	DDRA  = 0xFF ;
 	DDRB  = 0xFF ;
}

/*
	Initialisation des priphriques
*/
void init_devices(void) {
 	cli(); 						// inhibition des interruptions (pour avoir la paix)
 	port_init();				// initialisation de prts

	// MCU Control Register
	// - interruptions externes sur niveau bas de la ligne
	// - sleep mode dsactiv
 	MCUCR = 0x00;			
 	
 	// INT0 et INT1 inactives
 	GICR = 0x00;
 	
 	// activation de l'interrupt d'overflow du timer 0
 	TIMSK = 0x01; 	
 	
 	// rglage du pre-scaler du timer 0
 	TCCR0 = 0x03 ;	  
 	
 	sei();  	   	  			// autorisation des interruptions
}

//- --------------------------------------------------------------------------------
int main(void) {
	init_devices();
 
	 // rien  ajouter : tout est fait dans l'interrupt handler
	 return 0 ;
}

